/***************************************************************************
    begin                : Tuesday Nov 15 2005
    copyright            : (C) 2005 by Holger Danielsson
    email                : holger.danielsson@t-online.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
 
 
#ifndef KILEWIDGETSTATISTICS_H
#define KILEWIDGETSTATISTICS_H

#include <tqvariant.h>
#include <tqwidget.h>

class TQVBoxLayout;
class TQGridLayout;
class TQLabel;
class TQGroupBox;

class KileWidgetStatistics : public TQWidget
{
	Q_OBJECT
  

public:
	KileWidgetStatistics( TQWidget* parent = 0, const char* name = 0, WFlags fl = 0 );
	~KileWidgetStatistics();

	TQLabel* m_commentAboutHelp;
	TQLabel* m_warning;

	TQLabel* m_wordChar;
	TQLabel* m_commandChar;
	TQLabel* m_whitespaceChar;
	TQLabel* m_totalChar;
	TQLabel* m_wordCharText;
	TQLabel* m_commandCharText;
	TQLabel* m_whitespaceCharText;
	TQLabel* m_totalCharText;

	TQLabel* m_wordString;
	TQLabel* m_environmentString;
	TQLabel* m_commandString;
	TQLabel* m_totalString;
	TQLabel* m_wordStringText;
	TQLabel* m_environmentStringText;
	TQLabel* m_commandStringText;
	TQLabel* m_totalStringText;

	void updateColumns();

private:
	TQGridLayout *chargrouplayout;
	TQGridLayout *stringgrouplayout;


};

#endif // KILEWIDGETSTATISTICS_H
